@@ -6,6 +6,7 @@ from ._numerai_main import (
     fetch_numerai_tournament,
     fetch_numerai_example_predictions,
     fetch_numerai_example_validation_predictions,
+    fetch_numerai_feature_metadata,
 )
 
 __all__ = [
@@ -16,4 +17,5 @@ __all__ = [
     "fetch_numerai_tournament",
     "fetch_numerai_example_predictions",
     "fetch_numerai_example_validation_predictions",
+    "fetch_numerai_feature_metadata",
 ]
