@@ -54,11 +54,12 @@ https://github.com/numerai/example-scripts/blob/0a8c4f764a3aee3b7c1709058dd1488b
 https://github.com/uuazed/numerapi
 
 """
-from os.path import exists
+import json
+import logging
+from os.path import exists, splitext
 from os import makedirs, remove
 from typing import List
 
-import logging
 import pandas as pd
 from numerapi import NumerAPI
 from sklearn.datasets import get_data_home
@@ -67,7 +68,7 @@ from sklearn.utils import Bunch
 logger = logging.getLogger(__name__)
 
 
-def _get_numerai_fetcher(filename_float32, filename_int8, name):
+def _get_numerai_fetcher(filename_float32, filename_int8, name, has_rounds=False):
     """Return fetch function for passed files"""
 
     def fetch_numerai(
@@ -79,11 +80,20 @@ def _get_numerai_fetcher(filename_float32, filename_int8, name):
         as_frame=False,
         columns=None,
         int8=True,
+        round_num=None,
     ):
         # Make sure required columns will be read
         if columns:
             columns = list(set(columns + ["era", "data_type"]))
 
+        # Get round (if dataset supports rounds)
+        napi = NumerAPI()
+        if round_num and not has_rounds:
+            raise ValueError("`round_num` given for a dataset without rounds.")
+        if not round_num and has_rounds:
+            round_num = napi.get_current_round()
+            logger.info(f"Using current round={round_num}")
+
         # Get file locations
         data_home = get_data_home(data_home=data_home)
         if not exists(data_home):
@@ -91,6 +101,9 @@ def _get_numerai_fetcher(filename_float32, filename_int8, name):
         filename = filename_float32
         if int8:
             filename = filename_int8
+        if round_num:
+            suffix = "_" + str(round_num)
+            filename = _add_filename_suffix(filename, suffix)
         filepath = "/".join([data_home, filename])
 
         # Download and read dataset
@@ -99,7 +112,6 @@ def _get_numerai_fetcher(filename_float32, filename_int8, name):
                 raise IOError("Data not found and `download_if_missing` is False")
 
             logger.info(f"Downloading Numerai {name} data to {filepath}")
-            napi = NumerAPI()
             napi.download_dataset(filename, dest_path=filepath)
 
             df = pd.read_parquet(filepath, columns=columns)
@@ -469,6 +481,10 @@ def fetch_numerai_live(*args, **kwargs):
         If True, the feature columns will use the `int8` data type
         instead of `float32`. Target columns are always `float32`.
 
+    round_num : int, default=None
+        Tournament round to download. If None, current round will be
+        downloaded.
+
     Returns
     -------
     dataset : :class:`~sklearn.utils.Bunch`
@@ -529,6 +545,7 @@ def fetch_numerai_live(*args, **kwargs):
         "numerai_live_data.parquet",
         "numerai_live_data_int8.parquet",
         "live",
+        has_rounds=True,
     )
     return fetcher(*args, **kwargs)
 
@@ -567,6 +584,10 @@ def fetch_numerai_tournament(*args, **kwargs):
         If True, the feature columns will use the `int8` data type
         instead of `float32`. Target columns are always `float32`.
 
+    round_num : int, default=None
+        Tournament round to download. If None, current round will be
+        downloaded.
+
     Returns
     -------
     dataset : :class:`~sklearn.utils.Bunch`
@@ -627,11 +648,12 @@ def fetch_numerai_tournament(*args, **kwargs):
         "numerai_tournament_data.parquet",
         "numerai_tournament_data_int8.parquet",
         "tournament",
+        has_rounds=True,
     )
     return fetcher(*args, **kwargs)
 
 
-def _get_numerai_prediction_fetcher(filename, name):
+def _get_numerai_prediction_fetcher(fname, name, has_rounds=False):
     """Return prediction fetch function for passed files"""
 
     def fetch_numerai_predictions(
@@ -640,11 +662,24 @@ def _get_numerai_prediction_fetcher(filename, name):
         download_if_missing=True,
         return_y=False,
         as_frame=False,
+        round_num=None,
     ):
+        # Get round (if predictions support rounds)
+        napi = NumerAPI()
+        if round_num and not has_rounds:
+            raise ValueError("`round_num` given for predictions without rounds.")
+        if not round_num and has_rounds:
+            round_num = napi.get_current_round()
+            logger.info(f"Using current round={round_num}")
+
         # Get file locations
         data_home = get_data_home(data_home=data_home)
+        filename = fname
         if not exists(data_home):
             makedirs(data_home)
+        if round_num:
+            suffix = "_" + str(round_num)
+            filename = _add_filename_suffix(filename, suffix)
         filepath = "/".join([data_home, filename])
 
         # Download and read dataset
@@ -653,7 +688,6 @@ def _get_numerai_prediction_fetcher(filename, name):
                 raise IOError("Data not found and `download_if_missing` is False")
 
             logger.info(f"Downloading Numerai {name} predictions to {filepath}")
-            napi = NumerAPI()
             napi.download_dataset(filename, dest_path=filepath)
 
             df = pd.read_parquet(filepath)
@@ -703,6 +737,10 @@ def fetch_numerai_example_predictions(*args, **kwargs):
         If True, `prediction` and `id` are pandas Series. `frame` will
         be given.
 
+    round_num : int, default=None
+        Prediction round to download. If None, current round will be
+        downloaded.
+
     Returns
     -------
     dataset : :class:`~sklearn.utils.Bunch`
@@ -730,6 +768,7 @@ def fetch_numerai_example_predictions(*args, **kwargs):
     fetcher = _get_numerai_prediction_fetcher(
         "example_predictions.parquet",
         "example",
+        has_rounds=True,
     )
     return fetcher(*args, **kwargs)
 
@@ -781,6 +820,62 @@ def fetch_numerai_example_validation_predictions(*args, **kwargs):
     return fetcher(*args, **kwargs)
 
 
+def fetch_numerai_feature_metadata(*, data_home=None, download_if_missing=True):
+    """Load the Numerai feature metadata.
+
+    Parameters
+    ----------
+    data_home : str, default=None
+        Specify another download and cache folder for the feature
+        metadata. By default all data is stored in
+        `~/scikit_learn_data` subfolders.
+
+    download_if_missing : bool, default=True
+        If False, raise an IOError if the data is not locally available
+        instead of trying to download the data from the source bucket.
+
+    Returns
+    -------
+    feature_metadata : dict
+        Dictionary with the following keys.
+
+        feature_sets : dict
+            Dictionary containing lists of feature names as values.
+
+        feature_stats : dict
+            Dictionary with feature names as keys and a dictionary
+            of different statistics about each feature as values.
+
+    References
+    ----------
+
+    https://forum.numer.ai/t/october-2021-updates/4384
+
+    """
+    # Get file locations
+    data_home = get_data_home(data_home=data_home)
+    filename = "features.json"
+    if not exists(data_home):
+        makedirs(data_home)
+    filepath = "/".join([data_home, filename])
+
+    # Download and read dataset
+    if not exists(filepath):
+        if not download_if_missing:
+            raise IOError("Data not found and `download_if_missing` is False")
+
+        logger.info(f"Downloading Numerai feature metadata to {filepath}")
+        napi = NumerAPI()
+        napi.download_dataset(filename, dest_path=filepath)
+
+        feature_metadata_dict = _read_json_file(filepath)
+        remove(filepath)
+    else:
+        feature_metadata_dict = _read_json_file(filepath)
+
+    return feature_metadata_dict
+
+
 def _get_feature_names(df: pd.DataFrame) -> List[str]:
     """Get list of `df`s feature column names."""
     return [c for c in df if c.startswith("feature_")]
@@ -797,3 +892,17 @@ def _get_dtype(is_int8: bool) -> str:
         return "int8"
     else:
         return "float32"
+
+
+def _add_filename_suffix(filepath: str, suffix: str) -> str:
+    """Insert suffix between current filename and extension."""
+    filepath, file_extension = splitext(filepath)
+    return filepath + suffix + file_extension
+
+
+def _read_json_file(filepath: str) -> dict:
+    """Return content of json file at filepath."""
+    with open(filepath) as json_file:
+        content = json.load(json_file)
+
+    return content
